var Global_TimeClock = {
    dir: 'res/time_clock/'
};

var zMyTimeClock = cc.Node.extend({
    bgImage: null,
    barImage: null,
    progressBarImage: null,
    offsetX: 0,
    slidBallImage: null,
    timeWindowImage: null,
    maxTime: null,
    reduceDeltaTime: 0.1,
    direction: 'toR',
    timeWarning: 10,
    isTimeWarning: false,
    timeOutCallBack: null,
    timeOutTarget: null,
    ctor: function () {
        this._super();
    },
    // 设置背景
    setBgImage: function (dir, name) {
        if (!this.bgImage) {
            this.bgImage = zNewSpriteCreate(dir, name, this);
        } else {
            zSetSpriteTexture(dir, name, this.bgImage);
        }
    },
    // 设置背景位置
    setBGImagePos: function (pos) {
        this.bgImage.setPosition(pos);
    },
    // 设置进度条背景图
    setBarImage: function (dir, name) {
        if (!this.barImage) {
            this.barImage = zNewSpriteCreate(dir, name, this);
            var maskNode = new cc.ClippingNode();
            var stencil = new cc.Sprite(zGetFullPathPNG(dir, name));
            maskNode.setInverted(false);
            maskNode.setAlphaThreshold(0.5);
            maskNode.setStencil(stencil);
            this.addChild(maskNode);
            this.maskNode = maskNode;
            this.stencil = stencil;
        } else {
            zSetSpriteTexture(dir, name, this.barImage);
            zSetSpriteTexture(dir, name, this.stencil);
            // this.maskNode.setStencil(new cc.Sprite(zGetFullPath(dir, name)));
        }
    },
    // 设置进度条背景位置
    setBarImagePos: function (pos) {
        this.barImage.setPosition(pos);
        this.maskNode.setPosition(pos);
        this.offsetX = pos.x;
    },
    // 设置进度条图片
    setProgressBarImage: function (dir, name) {
        if (!this.progressBarImage) {
            this.progressBarImage = zNewSpriteCreate(dir, name, this.maskNode);
        }
    },
    // 设置当前位置进度球图片
    setSlidBallImage: function (dir, name) {
        if (!this.slidBallImage) {
            this.slidBallImage = zNewSpriteCreate(dir, name, this);
            this.slidBallImage.setPosition(this.progressBarImage.x + this.progressBarImage.width / 2, 0);
        } else {
            zSetSpriteTexture(dir, name, this.progressBarImage);
        }
    },
    getSlideBallImage: function () {
        return this.slidBallImage;
    },
    setTimeWindowImage: function (dir, name) {
        if (!this.timeWindowImage) {
            this.timeWindowImage = zNewSpriteCreate(dir, name, this);
            this.timeWindowImage.setPosition(-this.barImage.width / 2, 0);
            var scaleX = 150 / this.timeWindowImage.width;
            var scaleY = 100 / this.timeWindowImage.height;
            var scale = scaleX > scaleY ? scaleY : scaleX;
            this.timeWindowImage.setScale(scale);
            this.timeWindowImage.setPosition(-this.barImage.width / 2 - this.timeWindowImage.height * scale / 2 + this.offsetX, 0);

            this.timeWindowLabel = zNewLabelCreate('0', null, 40, this);
            this.timeWindowLabel.setPosition(this.timeWindowImage.getPosition());
        }
    },
    // 是否进行进度球动画
    runSlidBallImageAnimation: function (boolean, dt) {
        this.slidBallImage.stopAllActions();
        if (!boolean) {
            this.slidBallImage.setRotation(0);
            return;
        }
        dt = dt == undefined ? 0.1 : dt;
        this.slidBallImage.runAction(cc.sequence(
            cc.rotateTo(dt, -7).easing(cc.easeOut(2)),
            cc.rotateTo(dt, 0).easing(cc.easeIn(2)),
            cc.rotateTo(dt, 7).easing(cc.easeOut(2)),
            cc.rotateTo(dt, 0).easing(cc.easeIn(2))
        ).repeatForever());
    },
    // 设置进度条方向
    setDirection: function (direction) {
        if (this.direction != direction) {
            this.direction = direction;
        }
        if (this.direction == 'toL') {
            this.setRotation(180);
        } else if (this.direction == 'toU') {
            this.setRotation(-90);
            this.bgImage.setRotation(90);
            this.timeWindowImage.setRotation(90);
            this.timeWindowLabel.setRotation(90);
        } else if (this.direction == 'toD') {
            this.setRotation(90);
        } else {
            this.direction = 'toR';
            this.setRotation(0);
        }
    },
    // 设置最大时间
    setMaxTime: function (maxTime) {
        this.maxTime = maxTime;
    },
    // 设置当前时间 并刷新进度条和时间显示数字
    setCurTime: function (curTime) {
        if (curTime > this.maxTime) {
            this.curTime = this.maxTime;
        } else if (curTime <= 0) {
            this.curTime = 0;
            // zLog(this.curTime);
        } else {
            this.curTime = curTime;
        }
        // 0.9 是进度条的最大使用长度系数
        this.stencil.setPosition((-this.progressBarImage.width + this.progressBarImage.width * (this.curTime / this.maxTime)) * 0.9, 0);
        this.slidBallImage.setPosition(this.stencil.x + this.progressBarImage.width / 2 + this.offsetX, 0);
        if (Number(this.timeWindowLabel.getString()) != Math.ceil(this.curTime)) {
            // zLog('di da');
            this.timeWindowLabel.setString(Math.ceil(this.curTime));
        }
    },
    // 获取当前时间
    getCurTime: function () {
        return this.curTime;
    },
    // 设置每次时间减少量
    setReduceDeltaTime: function (deltaTime) {
        this.reduceDeltaTime = deltaTime;
    },
    // 减少时间
    reduceTime: function () {
        var curTime = this.curTime;
        if (curTime > 0) {
            curTime -= this.reduceDeltaTime;
            this.setCurTime(curTime);
            if (this.curTime < this.timeWarning && !this.isTimeWarning) {
                this.isTimeWarning = true;
                // zLog('time little !!!');
                this.runSlidBallImageAnimation(true, 0.05);
                zSetSpriteTexture(Global_TimeClock.dir, 'bar_time_little', this.progressBarImage);
                this.progressBarImage.setOpacity(0);
                this.progressBarImage.runAction(cc.fadeIn(0.5));
            } else if (this.curTime > this.timeWarning && this.isTimeWarning) {
                this.isTimeWarning = false;
                // zLog('time much');
                zSetSpriteTexture(Global_TimeClock.dir, 'bar_time_much', this.progressBarImage);
                this.runSlidBallImageAnimation(true, 0.1);
                this.progressBarImage.setOpacity(0);
                this.progressBarImage.runAction(cc.fadeIn(0.5));
            }
        } else {
            this.curTime = 0;
            this.stopTiming();
            // zLog('stop timing');
            if (this.timeOutCallBack && this.timeOutTarget) {
                this.timeOutCallBack.call(this.timeOutTarget);
            }
        }
    },
    // 开始计时
    startTiming: function () {
        this.schedule(this.reduceTime, this.reduceDeltaTime, cc.REPEAT_FOREVER, 0.1);
        this.runSlidBallImageAnimation(true);
    },
    // 停止计时
    stopTiming: function () {
        this.unschedule(this.reduceTime);
        this.runSlidBallImageAnimation(false);
    },
    setTimeOutCallBack: function (callBack, target) {
        this.timeOutCallBack = callBack;
        this.timeOutTarget = target;
    },
    rewardTime: function (time) {
        var self = this;
        var wp1 = cc.p(cc.winSize.width / 2, cc.winSize.height / 2);
        var time_add = zNewSpineAnimationCreate(Global_TimeClock.dir + 'spine/', 'skeleton', self);
        time_add.setRotation(90);
        time_add.setPosition(self.convertToNodeSpace(wp1));
        time_add.setTimeScale(1.5);
        time_add.setAnimation(0, time.toString(), false);
        time_add.setCompleteListener((trackEntry) => {
            var aniName = trackEntry.animation ? trackEntry.animation.name : '';
            if (aniName == '5' || aniName == '10') {
                time_add.runAction(cc.sequence(
                    cc.spawn(
                        zGetBezierTo2(0.5, time_add.getPosition(), self.getSlideBallImage().getPosition()), cc.scaleTo(0.5, 0.3)
                    ).easing(cc.easeIn(2)),
                    cc.callFunc(() => {
                        self.setCurTime(self.getCurTime() + time);
                        time_add.removeFromParent(true);
                    })
                ));
            }
        });
    }
});

function zAddMyTimeClock(parent, timeTotal, callBack) {
    var dir = Global_TimeClock.dir;
    var timeClock = new zMyTimeClock();
    timeClock.setLocalZOrder(100);
    parent.addChild(timeClock);
    timeClock.setPosition(-cc.winSize.width / 2 + 100, cc.winSize.height / 2);
    timeClock.setBgImage(dir, 'bg_left');
    timeClock.setBGImagePos(cc.p(0, 0));   // 因为可能经过旋转 x坐标对应的可能是高度的变化
    timeClock.setBarImage(dir, 'bar_bg');
    timeClock.setBarImagePos(cc.p(40, 0));
    timeClock.setProgressBarImage(dir, 'bar_time_much');
    timeClock.setSlidBallImage(dir, 'clock');
    timeClock.setTimeWindowImage(dir, 'time_window');
    timeClock.setDirection('toU');
    timeClock.setMaxTime(timeTotal);
    timeClock.setCurTime(timeTotal);
    timeClock.setReduceDeltaTime(0.01);
    timeClock.setTimeOutCallBack(callBack, parent);

    return timeClock;
}

// 倒计时节点
var zCountDown = cc.Node.extend({
    num: 0,
    num_min: 0,
    num_max: 0,
    num_image: null,
    num_image_dir: null,
    num_image_pr_name: null,
    countDownCallBack: null,
    countDownTarget: null,
    ctor: function () {
        this._super();
    },
    setNumRange: function (number_min, number_max) {
        this.num_min = number_min;
        this.num_max = number_max;
        this.num = this.num_max;
    },
    setNumImageDirAndPrName: function (dir, prName) {
        this.num_image_dir = dir;
        this.num_image_pr_name = prName;
    },
    addNumImage: function (parent, zOrder) {
        if (this.num_image) {
            this.num_image.removeFromParent(true);
        }
        this.num_image = zNewSpriteCreate(this.num_image_dir, this.num_image_pr_name + this.num, parent);
        this.num_image.setLocalZOrder(zOrder ? zOrder : 10);
        this.num_image.setScale(0);
    },
    setNumImagePos: function (pos) {
        this.num_image.setPosition(pos);
    },
    setNumImage: function (number) {
        this.num = number;
        zSetSpriteTexture(this.num_image_dir, this.num_image_pr_name + number, this.num_image);
    },
    runScaleImage: function () {
        this.num_image.stopAllActions();
        this.num_image.setScale(0);
        this.num_image.runAction(cc.sequence(
            cc.scaleTo(0.1, 1).easing(cc.easeIn(2)),
            cc.delayTime(0.5),
            cc.scaleTo(0.4, 0).easing(cc.easeOut(2))
        ));
    },
    setNumImageCallBack: function (callBack, target) {
        this.countDownCallBack = callBack;
        this.countDownTarget = target;
    },
    count: function () {
        // zLog('count : ', this.num);
        this.setNumImage(this.num);
        this.runScaleImage();
        var self = this;
        ddSound.playEffect(self.num > self.num_min ? 'time_short' : 'time_long');
        this.scheduleOnce(function () {
            self.num--;
            if (self.num >= self.num_min) {
                self.count();
            } else {
                self.countDownCallBack.call(self.countDownTarget);
            }
        }, 1.1);
    }
});

function zAddCountDown(parent, number_min, number_max, callBack, target) {
    var count_down = new zCountDown();
    parent.addChild(count_down);
    count_down.setNumRange(number_min, number_max);
    count_down.setNumImageDirAndPrName(Global_TimeClock.dir, 'count_down_');
    count_down.addNumImage(parent, 50);
    count_down.setNumImagePos(cc.p(0, cc.winSize.height / 2));
    count_down.setNumImageCallBack(callBack, target);
    return count_down;
}